<?php

declare(strict_types=1);

namespace apexl\Io\modules\system\interfaces;

use apexl\Io\includes\RouteManager;

interface ProvidesRoutesInterface
{
    public function routes(RouteManager $routeManager): void;
}