<?php

use function apexl\io\env;

return [
    'host' => env('SMTP_HOST', 'localhost'),
    'auth' => env('SMTP_AUTH', false),
    'username' => env('SMTP_USERNAME', ''),
    'password' => env('SMTP_PASSWORD', ''),
    'port' => env('SMTP_PORT', '1025'),
    'from' => env('SMTP_FROM', 'noreply@example.com'),
    'encryption' => env('SMTP_ENCRYPTION', false),
];